local AddOn, NS = ...
local strfind = string.find

local function DUF_Element_SetColor(self, r, g, b, alpha)
	if (self.index == "StatusBar") then
		self.Bar1:SetVertexColor(r, g, b)
	elseif (self.index == "TextBox") then
		self.TextString:SetTextColor(r, g, b)
	else
		self.Background:SetBackdropColor(r, g, b, alpha)
	end
end

local function DUF_Element_GetOffsets(element, baseframe, settings)
	if (settings.attach ~= "Unit Frame" and settings.attach ~= "UIParent") then
		baseframe = _G[baseframe:GetName()..settings.attach]
	elseif (settings.attach == "UIParent") then
		baseframe = UIParent
	end
	local xoffset, yoffset = DL_Get_Offsets(element, baseframe, settings.attachpoint, settings.attachto)
	return xoffset, yoffset
end

------------------------------------------------------- GLOBAL FUNCTIONS -----------------------------------------

function DUF_Element_OnDragStart(self)
	if (DUF_ELEMENTS_UNLOCKED) then
		self.moving = true
		if (self.moveparent) then
			self:GetParent():StartMoving()
		elseif (self.moveparentparent) then
			self:GetParent():GetParent():StartMoving()
		else
			self:StartMoving()
		end
	elseif (DUF_FRAMES_UNLOCKED) then
		if (self.moveparent) then
			self:GetParent():GetParent():StartMoving()
		elseif (self.moveparentparent) then
			self:GetParent():GetParent():GetParent():StartMoving()
		else
			self:GetParent():StartMoving()
		end
	end
end

function DUF_Element_OnDragStop(self)
	if (DUF_ELEMENTS_UNLOCKED) then
		self.moving = nil
		if (self.moveparent) then
			self:GetParent():StopMovingOrSizing()
		elseif (self.moveparentparent) then
			self:GetParent():GetParent():StopMovingOrSizing()
		else
			self:StopMovingOrSizing()
		end
		local frameindex, unitframe, xoffset, yoffset, settings
		if (self.moveparent) then
			frameindex = DUF_FRAME_DATA[self:GetParent():GetParent().unit].index
			if (self:GetParent().subindex) then
				settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent().index][self:GetParent().subindex]
			else
				settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent().index]
			end
			unitframe = _G[self:GetParent():GetParent():GetName()]
			xoffset, yoffset = DUF_Element_GetOffsets(self:GetParent(), unitframe, settings)
		elseif (self.moveparentparent) then
			frameindex = DUF_FRAME_DATA[self:GetParent():GetParent():GetParent().unit].index
			if (self:GetParent():GetParent().subindex) then
				settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent():GetParent().index][self:GetParent().subindex]
			else
				settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent():GetParent().index]
			end
			unitframe = _G[self:GetParent():GetParent():GetParent():GetName()]
			xoffset, yoffset = DUF_Element_GetOffsets(self:GetParent():GetParent(), unitframe, settings)
		else
			frameindex = DUF_FRAME_DATA[self:GetParent().unit].index
--			frameindex = DUF_FRAME_DATA[self.unit].index
			if (self.subindex) then
				settings = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex]
			else
				settings = DUF_Settings[DUF_INDEX][frameindex][self.index]
			end
			unitframe = _G[self:GetParent():GetName()]
			xoffset, yoffset = DUF_Element_GetOffsets(self, unitframe, settings)
		end
		if (self.subindex) then
			DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].xoffset = xoffset
			DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].yoffset = yoffset
		elseif (self.moveparent) then
			DUF_Settings[DUF_INDEX][frameindex][self:GetParent().index].xoffset = xoffset
			DUF_Settings[DUF_INDEX][frameindex][self:GetParent().index].yoffset = yoffset
		elseif (self.moveparentparent) then
			DUF_Settings[DUF_INDEX][frameindex][self:GetParent():GetParent().index].xoffset = xoffset
			DUF_Settings[DUF_INDEX][frameindex][self:GetParent():GetParent().index].yoffset = yoffset
		else
			DUF_Settings[DUF_INDEX][frameindex][self.index].xoffset = xoffset
			DUF_Settings[DUF_INDEX][frameindex][self.index].yoffset = yoffset
		end
		if (self.moveparent) then
			local settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent().index]
			local element = DUF_Get_ElementName(self:GetParent().index, self:GetParent().subindex)
			if (frameindex == "party" or frameindex == "partypet") then
				for i=1,4 do
					local unitFrame = DUF_FRAME_DATA[frameindex..i].frame
					NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
				end
			elseif frameindex == "boss" then
				for i=1,5 do
					local unitFrame = DUF_FRAME_DATA[frameindex..i].frame
					NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
				end
			else
				local unitFrame = DUF_FRAME_DATA[frameindex].frame
				NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
			end
		elseif (self.moveparentparent) then
			local settings = DUF_Settings[DUF_INDEX][frameindex][self:GetParent():GetParent().index]
			local element = DUF_Get_ElementName(self:GetParent():GetParent().index, self:GetParent():GetParent().subindex)
			if (frameindex == "party" or frameindex == "partypet") then
				for i=1,4 do
					local unitFrame = DUF_FRAME_DATA[frameindex..i].frame
					NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
				end
			elseif frameindex == "boss" then
				for i=1,5 do
					local unitFrame = DUF_FRAME_DATA[frameindex..i].frame
					NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
				end
			else
				local unitFrame = DUF_FRAME_DATA[frameindex].frame
				NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
			end
		else
			local settings
			if (self.subindex) then
				settings = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex]
			else
				settings = DUF_Settings[DUF_INDEX][frameindex][self.index]
			end
			local element = DUF_Get_ElementName(self.index, self.subindex)
			if (frameindex == "party" or frameindex == "partypet" or frameindex == "boss") then
				local counter = 4
				if frameindex == "boss" then
					counter = 5
				end
				for i=1,counter do
					local unitFrame = DUF_FRAME_DATA[frameindex..i].frame
					NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
				end
			else
				local unitFrame = DUF_FRAME_DATA[frameindex].frame
				NS:DUF_Set_ElementPosition(_G[unitFrame..element], unitFrame, settings)
			end
		end
		if (DUF_Options) then
			DUF_Init_BaseOptions()
		end
	elseif (DUF_FRAMES_UNLOCKED) then
		if (self.moveparent) then
			DUF_UnitFrame_OnDragStop(self:GetParent():GetParent())
		elseif (self.moveparentparent) then
			DUF_UnitFrame_OnDragStop(self:GetParent():GetParent():GetParent())
		else
			DUF_UnitFrame_OnDragStop(self:GetParent())
		end
	end
end

local function SetActiveInactive(frame, active, inactive)
	   	if frame.hidewhensolo and (GetNumGroupMembers() + GetNumSubgroupMembers()) == 0 then
	   		return
	   	end
		frame:SetScript("OnUpdate", function(self, elapsed)
		local current = GetMouseFocus()
		while current ~= nil do
			if current == frame then
				frame:SetAlpha(active)
				return
			end
			current = current:GetParent()
		end
		frame:SetAlpha(inactive)
		frame:SetScript("OnUpdate", nil)
	end)    
end

function DUF_Element_OnEnter(self)
	if (self.index == nil) then return end
	local unit = self:GetParent().unit
	local settings
	if (self.subindex) then
		settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index][self.index][self.subindex]
	else
		settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index][self.index]
	end
	if (settings.usemouseovercolor) then
		if settings.mouseovercolor.r == nil then -- fix for change in SetVertecColor not accepting nil
			settings.mouseovercolor.r = 0
			settings.mouseovercolor.g = 0
			settings.mouseovercolor.b = 0
		end
		self:SetBackdropColor(settings.mouseovercolor.r, settings.mouseovercolor.g, settings.mouseovercolor.b, settings.bgalpha)
	end
	if settings.hidemouseover then
		SetActiveInactive(self, 0, settings.alpha)
	end
	if settings.mouseover then
		SetActiveInactive(self, settings.alpha, 0)
	end
	if (settings.mouseovergroup) then
		local unitFrame = self:GetParent()
		local unitFrameName = unitFrame:GetName()
		local element

		if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].mouseovergroup == settings.mouseovergroup) then
			if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].usemouseovercolor) then
				_G[self:GetParent():GetName()].Background:SetBackdropColor(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].mouseovercolor.r, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].mouseovercolor.g, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].mouseovercolor.b, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].bgalpha)
			end
		end

		for index, value in pairs(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]) do
			if (type(value) == "table") then
				if (index == "StatusBar") then
					for i = 1,6 do
						if (value[i]) then
							if (value[i].mouseovergroup == settings.mouseovergroup) then
								if (value[i].mouseover) then
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(value[i].alpha)
									SetActiveInactive(self, settings.alpha, 0)
									
								elseif (value[i].hidemouseover) then
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
									SetActiveInactive(self, 0, settings.alpha)
								end
								if (value[i].usemouseovercolor) then
									_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].mouseovercolor.r, value[i].mouseovercolor.g, value[i].mouseovercolor.b, value[i].bgalpha)
								end
							end
						end
					end
				elseif (index == "TextBox") then
					local tb
					if (unit == "player") then
						tb = NS.DUF_NUM_PLAYER_TEXTBOXES
					else
						tb = NS.DUF_NUM_OTHERS_TEXTBOXES
					end
					for i = 1,tb do
						if (value[i].mouseovergroup == settings.mouseovergroup) then
							if (value[i].mouseover) then
--								_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(1)
							elseif (value[i].hidemouseover) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
							end
							if (value[i].usemouseovercolor) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].mouseovercolor.r, value[i].mouseovercolor.g, value[i].mouseovercolor.b, value[i].bgalpha)
							end
						end
					end
				else
					if (value.mouseovergroup == settings.mouseovergroup) then
						if (value.mouseover) then
--							_G[unitFrameName..DUF_Get_ElementName(index, i)]:Show()
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(1)
						elseif (value.hidemouseover) then
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
						end
						if (value.usemouseovercolor) then
							_G[unitFrameName..DUF_Get_ElementName(index)].Background:SetBackdropColor(value.mouseovercolor.r, value.mouseovercolor.g, value.mouseovercolor.b, value.bgalpha)
						end
					end
				end
			end
		end
	end

--	if (settings.hidemouseover) then
--		self.oldalpha = self:GetAlpha()
--		self:SetAlpha(0)
--	end

	if (settings.disabletooltip) then
		return
	end
	if (unit and UnitName(unit)) then
		if (settings.usecustomtooltip) then
			local text = settings.customtooltip
			if (text) then
				for var, value in pairs(NS.DUF_VARIABLE_FUNCTIONS) do
					if (strfind(text, var)) then
						text = value.func(text, unit, self) 
					end
				end
				GameTooltip_SetDefaultAnchor(GameTooltip, self)	
				GameTooltip:SetText(text, 1, 1, 1, 1, 1)
				GameTooltip:Show()
			end
		else
			self.unit = unit
	 		UnitFrame_OnEnter(self)
		end
	end
end

function DUF_Element_OnHide(self)
	if (self.moving) then
		DUF_Element_OnDragStop(self)
	end
end

function DUF_Element_OnLeave(self)
	if (self:GetParent().unit and UnitName(self:GetParent().unit)) then
		self.unit = self:GetParent().unit
		UnitFrame_OnLeave(self)
	end
	if (self.moving) then
		DUF_Element_OnDragStop(self)
	end
	if (self.index == nil) then return end
	local unit = self:GetParent().unit
	local settings
	if (self.subindex) then
		settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index][self.index][self.subindex]
	else
		settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index][self.index]
	end
	if (self.oldalpha) then
		self:SetAlpha(self.oldalpha)
		self.oldalpha = nil
	end
	if (settings.usemouseovercolor) then
		self:SetBackdropColor(settings.bgcolor.r, settings.bgcolor.g, settings.bgcolor.b, settings.bgalpha)
	end
--	if settings.hidemouseover then
--		self:SetScript("OnUpdate", nil)
--		self:SetAlpha(settings.alpha)
--	end
--	if settings.mouseover then
--		self:SetScript("OnUpdate", nil)
--		self:SetAlpha(0)
--	end
	if (settings.mouseovergroup) then
		local unitFrame = self:GetParent()
		local unitFrameName = unitFrame:GetName()
		local element

		if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].mouseovergroup == settings.mouseovergroup) then
			if (DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].usemouseovercolor) then
				_G[self:GetParent():GetName()].Background:SetBackdropColor(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].bgcolor.r, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].bgcolor.g, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].bgcolor.b, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].bgalpha)
			end
		end

		for index, value in pairs(DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]) do
			if (type(value) == "table") then
				if (index == "StatusBar") then
--					for i = 1,6 do
					for i = 1, NS.DUF_NUM_STATUSBARS do
						if (value[i]) then
							if (value[i].mouseovergroup == settings.mouseovergroup) then
--								if (value[i].mouseover) then
----									_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
--								elseif (value[i].hidemouseover) then
--									_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(value[i].alpha)
--								end
								if (value[i].usemouseovercolor) then
									_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].bgcolor.r, value[i].bgcolor.g, value[i].bgcolor.b, value[i].bgalpha)
								end
							end
						end
					end
				elseif (index == "TextBox") then
					local Count = NS.DUF_NUM_OTHERS_TEXTBOXES
					if unit == "player" then
						Count = NS.DUF_NUM_PLAYER_TEXTBOXES
					end
--					for i = 1,10 do
					for i = 1, Count do
						if (value[i].mouseovergroup == settings.mouseovergroup) then
							if (value[i].mouseover) then
--								_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
							elseif (value[i].hidemouseover) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(value[i].alpha)
							end
							if (value[i].usemouseovercolor) then
								_G[unitFrameName..DUF_Get_ElementName(index, i)].Background:SetBackdropColor(value[i].bgcolor.r, value[i].bgcolor.g, value[i].bgcolor.b, value[i].bgalpha)
							end
						end
					end
				else
					if (value.mouseovergroup == settings.mouseovergroup) then
						if (value.mouseover) then
--							_G[unitFrameName..DUF_Get_ElementName(index, i)]:Hide()
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(0)
						elseif (value.hidemouseover) then
							_G[unitFrameName..DUF_Get_ElementName(index, i)]:SetAlpha(value.alpha)
						end
						if (value.usemouseovercolor) then
							_G[unitFrameName..DUF_Get_ElementName(index)].Background:SetBackdropColor(value.bgcolor.r, value.bgcolor.g, value.bgcolor.b, value.bgalpha)
						end
					end
				end
			end
		end
	end
end

function DUF_Element_OnLoad(self)
	local use3D
	local unit = self:GetParent().unit
	local name = self:GetName()
	if (strfind(name, "Portrait")) then
		use3D = NS:DUF_Get3D(unit)
	end
	self:RegisterForDrag("LeftButton", "RightButton")
	local type = self:GetObjectType()
	if type == "Button" then 
		ClickCastFrames = ClickCastFrames or {}
		self:RegisterForClicks("AnyDown")
		ClickCastFrames[self] = true
	end	
	self:SetAttribute("useparent*",true)
	if (strfind(name, "Bar")) then
		self.index = "StatusBar"
		self.subindex = self:GetID()
	elseif (strfind(name, "Portrait")) then
		self.index = "Portrait"
	elseif (strfind(name, "ClassIcon")) then
		self.index = "ClassIcon"
	elseif (strfind(name, "RankIcon")) then
		self.index = "RankIcon"
	elseif (strfind(name, "RaceIcon")) then
		self.index = "RaceIcon"
	elseif (strfind(name, "Buffs")) then
		self.index = "Buffs"
	elseif (strfind(name, "RoleIcon")) then
		self.index = "RoleIcon"
	elseif (strfind(name, "Debuffs")) then
		self.index = "Debuffs"
	elseif (strfind(name, "LootIcon")) then
		self.index = "LootIcon"
	elseif (strfind(name, "PVPIcon")) then
		self.index = "PVPIcon"
	elseif (strfind(name, "LeaderIcon")) then
		self.index = "LeaderIcon"
	elseif (strfind(name, "StatusIcon")) then
		self.index = "StatusIcon"
	elseif (strfind(name, "TextBox")) then
		self.index = "TextBox"
		self.subindex = self:GetID()
	elseif (strfind(name, "HappinessIcon")) then
--		if not GetPetHappiness then -- GetPetHappiness function no longer exists
--			DUF_PetFrame_HappinessIcon:SetScript("OnEnter", nil)
--		end
	elseif (strfind(name, "EliteTexture")) then
		self.index = "EliteTexture"
	elseif (strfind(name, "Markers")) then
		self.index = "Markers"
	elseif (strfind(name, "QuestBossIcon")) then
		self.index = "QuestBossIcon"
	elseif (strfind(name, "TempEnchants")) then
		self.index = "TempEnchants"
	end
	self.baseframelevel = self:GetFrameLevel()
end

function DUF_Element_OnShow(self)
	if (not DUF_INITIALIZED) then return end
	local FrameName = self:GetName()
	if not FrameName then return end
	local unit = self:GetParent().unit
	local frameindex = DUF_FRAME_DATA[unit].index
--	if (strfind(self:GetName(), "TargetHealthBar") or strfind(self:GetName(), "TargetManaBar") or self.checkname) then
	if (strfind(FrameName, "TargetHealthBar") or strfind(FrameName, "TargetManaBar") or self.checkname) then
		unit = unit.."target"
	end
	local color, bgcolor, bordercolor
	local context, bgcontext, bordercontext
	local settings = DUF_Settings[DUF_INDEX][frameindex][self.index]
	if (self.subindex) then
		context = settings[self.subindex].context
		bgcontext = settings[self.subindex].bgcontext
		bordercontext = settings[self.subindex].bordercontext
	elseif (self.index) then
		context = settings.context
		bgcontext = settings.bgcontext
		bordercontext = settings.bordercontext
	else
		return
	end
	if ((not context) and (not bgcontext) and (not bordercontext)) then return end
	if (self.subindex) then
		bgcolor = settings[self.subindex].bgcolor
		bordercolor = settings[self.subindex].bordercolor
		bgalpha = settings[self.subindex].bgalpha
		borderalpha = settings[self.subindex].borderalpha
	else
		bgcolor = settings.bgcolor
		bordercolor = settings.bordercolor
		bgalpha = settings.bgalpha
		borderalpha = settings.borderalpha
	end
	local background = self.Background
	if self.index == "StatusBar" then
--		if (self.subindex == 2 or self.subindex == 5) then
		if self.subindex == NS.Bar.Power or self.subindex == NS.Bar.TargetPower then
			color = self.manacolor
		else
			color = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].color
		end
	elseif (self.index == "TextBox") then
		color = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].textcolor
	else
		color = bgcolor
	end
	local r, g, b
	if (context and context < 4) then
		if (context == 1) then
			r, g ,b = NS:DUF_Get_DifficultyColor(unit, color)
		elseif (context == 2) then
			r, g ,b = NS:DUF_Get_ReactionColor(unit, color)
		elseif (context == 3) then
			r, g ,b = NS:DUF_Get_ClassColor(unit, color)
		end
		DUF_Element_SetColor(self, r, g, b, bgalpha)
	end
	if (bgcontext and bgcontext < 4) then
		if (bgcontext == 1) then
			r, g ,b = NS:DUF_Get_DifficultyColor(unit, bgcolor)
		elseif (bgcontext == 2) then
			r, g ,b = NS:DUF_Get_ReactionColor(unit, bgcolor)
		elseif (bgcontext == 3) then
			r, g ,b = NS:DUF_Get_ClassColor(unit, bgcolor)
		end
		background:SetBackdropColor(r, g, b, bgalpha)
	end
	if (bordercontext and bordercontext < 4) then
		if (bordercontext == 1) then
			r, g ,b = NS:DUF_Get_DifficultyColor(unit, color)
		elseif (bordercontext == 2) then
			r, g ,b = NS:DUF_Get_ReactionColor(unit, color)
		elseif (bordercontext == 3) then
			r, g ,b = NS:DUF_Get_ClassColor(unit, color)
		end
		background:SetBackdropBorderColor(r, g, b, borderalpha)
	end
------------ Fizz Added to update on DUF Config close (replicates OnUpdate code without the status check)
	if (context == 4 or bgcontext == 4 or bordercontext == 4) then
		local health = UnitHealth(unit)
		local healthMax = UnitHealthMax(unit)
		self.health = health
		self.healthmax = healthMax
		if (context == 4) then
			r, g, b = NS:DUF_Get_HealthColor(unit, color, self.health, self.healthmax)
			DUF_Element_SetColor(self, r, g, b, bgalpha)
		end
		if (bgcontext == 4) then
			r, g, b = NS:DUF_Get_HealthColor(unit, bgcolor, self.health, self.healthmax)
			self.Background:SetBackdropColor(r, g, b, bgalpha)
		end
		if (bordercontext == 4) then
			r, g, b = NS:DUF_Get_HealthColor(unit, bordercolor, self.health, self.healthmax)
			self.Background:SetBackdropBorderColor(r, g, b, borderalpha)
		end
	end
	if (context == 5 or bgcontext == 5 or bordercontext == 5) then
		local mana = UnitPower(unit)
		local manamax = UnitPower(unit)
		self.mana = mana
		self.manamax = manamax
		if (context == 5) then
			if (not color) then return end
			r, g, b = NS:DUF_Get_ManaColor(unit, color)
			DUF_Element_SetColor(self, r, g, b, bgalpha)
		end
		if (bgcontext == 5) then
			r, g, b = NS:DUF_Get_ManaColor(unit, bgcolor)
			self.Background:SetBackdropColor(r, g, b, bgalpha)
		end
		if (bordercontext == 5) then
			r, g, b = NS:DUF_Get_ManaColor(unit, bordercolor)
			self.Background:SetBackdropBorderColor(r, g, b, borderalpha)
		end
	end
end

function DUF_Element_OnUpdate(self, elapsed)
	if (not DUF_INITIALIZED or not self:IsVisible() or self:GetAlpha() == 0) then return end
	if (not self.index) then return end
	if self.index == QuestBossIcon then return end
	if (self.flashing) then
		self.flashtime = self.flashtime - elapsed
		if (self.flashtime < 0) then
			self.flashtime = .5
			if (self.direction) then
				self.direction = nil
			else
				self.direction = 1
			end
		end
		if (self.direction) then
			self:SetAlpha(1 - self.flashtime)
		else
			self:SetAlpha(self.flashtime + .5)
		end
	end
	if (not self.timer) then 
		self.timer = DUF_Settings[DUF_INDEX].updatespeed 
		self.updatespeed = self.timer
	end
	self.timer = self.timer - elapsed
	if (self.timer < 0) then
		self.timer = self.updatespeed
		if (self.index == "TextBox") then
			self.update = true
		end
		local unit = self:GetParent().unit
		local frameindex = DUF_FRAME_DATA[unit].index
		if (strfind(self:GetName(), "TargetHealthBar") or strfind(self:GetName(), "TargetManaBar") or self.checkname) then
			unit = unit.."target"
		end
		local context, bgcontext, bordercontext, flash, targetIcon
		if (self.subindex) then
			context = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].context
			bgcontext = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].bgcontext
			bordercontext = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].bordercontext
			flash = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].flashlowhealth
		else
			context = DUF_Settings[DUF_INDEX][frameindex][self.index].context
			bgcontext = DUF_Settings[DUF_INDEX][frameindex][self.index].bgcontext
			bordercontext = DUF_Settings[DUF_INDEX][frameindex][self.index].bordercontext
			flash = DUF_Settings[DUF_INDEX][frameindex][self.index].flashlowhealth
		end
		if (flash) then
			local health = UnitHealth(unit)
			local healthMax = UnitHealthMax(unit)
			if health and healthMax then
				local unithealth
				if healthMax == 0 then
					unithealth = health
				else
					unithealth = health / healthMax
				end
				if (unithealth <= DUF_Settings[DUF_INDEX].flashthreshold and health > 0) then
					if (not self.flashing) then
						self.flashing = true
						self.direction = 1
						self.flashtime = .5
					end
				else
					self.flashing = nil
					if (self.subindex) then
						self:SetAlpha(DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].alpha)
					else
						self:SetAlpha(DUF_Settings[DUF_INDEX][frameindex][self.index].alpha)
					end
				end
			end
		end
--------
		if ((not context) and (not bgcontext) and (not bordercontext)) then return end
		local color, bgcolor, bordercolor
		if (self.subindex) then
			bgcolor = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].bgcolor
			bordercolor = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].bordercolor
			bgalpha = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].bgalpha
			borderalpha = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].borderalpha
		else
			bgcolor = DUF_Settings[DUF_INDEX][frameindex][self.index].bgcolor
			bordercolor = DUF_Settings[DUF_INDEX][frameindex][self.index].bordercolor
			bgalpha = DUF_Settings[DUF_INDEX][frameindex][self.index].bgalpha
			borderalpha = DUF_Settings[DUF_INDEX][frameindex][self.index].borderalpha
		end
		if self.index == "StatusBar" then
--			if (self.subindex == 2 or self.subindex == 5) then
			if self.subindex == NS.Bar.Power or self.subindex == NS.Bar.TargetPower then
				color = self.manacolor
			else
				color = DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].color
			end
		elseif (self.index == "TextBox") then
			color =  DUF_Settings[DUF_INDEX][frameindex][self.index][self.subindex].textcolor
		else
			color = bgcolor
		end
		local r, g, b
		if (context == 4 or bgcontext == 4 or bordercontext == 4) then
			local health = UnitHealth(unit)
			local healthMax = UnitHealthMax(unit)
			if (self.health ~= health or self.healthmax ~= self.healthmax) then
				self.health = health
				self.healthmax = healthMax
				if (context == 4) then
					r, g, b = NS:DUF_Get_HealthColor(unit, color, self.health, self.healthmax)
					DUF_Element_SetColor(self, r, g, b, bgalpha)
				end
				if (bgcontext == 4) then
					r, g, b = NS:DUF_Get_HealthColor(unit, bgcolor, self.health, self.healthmax)
					self.Background:SetBackdropColor(r, g, b, bgalpha)
				end
				if (bordercontext == 4) then
					r, g, b = NS:DUF_Get_HealthColor(unit, bordercolor, self.health, self.healthmax)
					self.Background:SetBackdropBorderColor(r, g, b, borderalpha)
				end
			end
		end
		if (context == 5 or bgcontext == 5 or bordercontext == 5) then
			local mana = UnitPower(unit)
			local manamax = UnitPower(unit)
			if (self.mana ~= mana or self.manamax ~= self.manamax) then
				self.mana = mana
				self.manamax = manamax
				if (context == 5) then
					if (not color) then return end
					r, g, b = NS:DUF_Get_ManaColor(unit, color)
					DUF_Element_SetColor(self, r, g, b, bgalpha)
				end
				if (bgcontext == 5) then
					r, g, b = NS:DUF_Get_ManaColor(unit, bgcolor)
					self.Background:SetBackdropColor(r, g, b, bgalpha)
				end
				if (bordercontext == 5) then
					r, g, b = NS:DUF_Get_ManaColor(unit, bordercolor)
					self.Background:SetBackdropBorderColor(r, g, b, borderalpha)
				end
			end
		end
		if (context == 2 or bgcontext == 2 or bordercontext == 2) then
			local reaction = NS:DUF_Get_Reaction(unit)
			if (self.reaction ~= reaction) then
				self.reaction = reaction
				if (context == 2) then
					if (not color) then return end
					r, g, b = NS:DUF_Get_ReactionColor(unit, color)
					DUF_Element_SetColor(self, r, g, b, bgalpha)
				end
				if (bgcontext == 2) then
					r, g, b = NS:DUF_Get_ReactionColor(unit, bgcolor)
					self.Background:SetBackdropColor(r, g, b, bgalpha)
				end
				if (bordercontext == 2) then
					r, g, b = NS:DUF_Get_ReactionColor(unit, bordercolor)
					self.Background:SetBackdropBorderColor(r, g, b, borderalpha)
				end
			end
		end
	end
end

